varying highp vec2 coordinate;
uniform sampler2D motionTexture;
uniform sampler2D inverseMotionTexture;
uniform sampler2D framePrev;
uniform sampler2D frameNext;
uniform highp float deltaTime;


const highp float textureUniformEncodeScale = 4.0;


const highp float EPSILON = 0.00390625; // 2^-8


//#define OUTPUT_INPUT_ONLY
//#define OUTPUT_FLOW_ONLY
//#define OUTPUT_INTERPOLATION_BLEND_WITH_FLOW
#define OUTPUT_INTERPOLATION_ONLY



highp vec2 decodeTextureUniformPerFrame( highp vec4 encodedPixelsPerFrame )
{
    highp vec4 textureUniformPerFrame = encodedPixelsPerFrame / textureUniformEncodeScale;
    
    highp vec2 positive = textureUniformPerFrame.xy;
    highp vec2 negative = textureUniformPerFrame.zw;
    
    return positive - negative;
}



void main()
{
    highp vec4 flowEncoded = texture2D( motionTexture, coordinate );
    
#ifdef OUTPUT_INPUT_ONLY
    
    gl_FragColor = flowEncoded;
    
#else // OUTPUT_FLOW_ONLY, OUTPUT_INTERPOLATION_ONLY, OUTPUT_INTERPOLATION_BLEND_WITH_FLOW
    
    highp vec2 flow = decodeTextureUniformPerFrame( flowEncoded );
    
# ifndef OUTPUT_INTERPOLATION_ONLY // OUTPUT_FLOW_ONLY, OUTPUT_INTERPOLATION_BLEND_WITH_FLOW
    
    //    highp vec4 flowScaled = vec4( normalize( abs( flow ) ), 0.0, 1.0 );
    highp vec4 flowScaled = vec4( abs( flow * vec2( 160, 90 ) / 16.0 ), 0.0, 1.0 );
    //    highp vec4 flowScaled = vec4( flow * (vec2(240,135)/32.0) /*vec2( 128, 72 )*/ + vec2(0.5,0.5), 0.0, 1.0 );
    
# endif // not OUTPUT_INTERPOLATION_ONLY, OUTPUT_FLOW_ONLY, OUTPUT_INTERPOLATION_BLEND_WITH_FLOW
    
# ifdef OUTPUT_FLOW_ONLY
    
    gl_FragColor = flowScaled;
    
# else // OUTPUT_INTERPOLATION_ONLY, OUTPUT_INTERPOLATION_BLEND_WITH_FLOW
    
    /*
    highp vec4 prevOrigonalColor = texture2D(framePrev,coordinate);
    highp vec4 nextOrigonalColor = texture2D(frameNext,coordinate);
    highp vec4 colorDifference = texture2D(framePrev,coordinate) - texture2D(frameNext,coordinate);
    if(dot(colorDifference,colorDifference)<EPSILON)
    {
        gl_FragColor = (1.0-deltaTime) * prevOrigonalColor + deltaTime * nextOrigonalColor;
        return;
    }
     */
    
    highp vec2 inverseFlow = decodeTextureUniformPerFrame( texture2D( inverseMotionTexture, coordinate ) );
    
    highp vec4 pixelBlend = (1.0-deltaTime) * texture2D(framePrev, coordinate - deltaTime * flow) + deltaTime * texture2D( frameNext, coordinate - (1.0-deltaTime) * inverseFlow);
//	highp vec4 pixelBlend = vec4(-inverseFlow.x,0.0,0.0,1.0);
//    highp vec4 pixelBlend =  texture2D( frameNext, coordinate - (1.0-deltaTime) * inverseFlow);
    
#  ifdef OUTPUT_INTERPOLATION_BLEND_WITH_FLOW
    
    if( dot( flowScaled, flowScaled ) < EPSILON )
        
        gl_FragColor = pixelBlend;
    
    else
        
        gl_FragColor = ( pixelBlend + flowScaled ) / 2.0;
    
#  else // OUTPUT_INTERPOLATION_ONLY
    
    gl_FragColor = pixelBlend;
    
#  endif // OUTPUT_INTERPOLATION_BLEND_WITH_FLOW, OUTPUT_INTERPOLATION_ONLY
    
# endif // OUTPUT_FLOW_ONLY, OUTPUT_INTERPOLATION_ONLY, OUTPUT_INTERPOLATION_BLEND_WITH_FLOW
    
#endif // OUTPUT_INPUT_ONLY, OUTPUT_FLOW_ONLY, OUTPUT_INTERPOLATION_ONLY, OUTPUT_INTERPOLATION_BLEND_WITH_FLOW
}

